@@ -1,6 +1,6 @@
 """irafpar.py -- parse IRAF .par files and create lists of IrafPar objects
 
-$Id: irafpar.py 1935 2013-02-26 16:41:36Z sontag $
+$Id: irafpar.py 2080 2013-11-06 19:31:41Z sontag $
 
 R. White, 2000 January 7
 """
@@ -508,6 +508,8 @@ class IrafParList(taskpars.TaskPars):
         """
         self.__pars = []
         self.__hasPsets = False
+        self.__psets2merge = None # is a list when populated
+        self.__psetLock = False
         self.__filename = filename
         self.__name = taskname
         self.__filecache = ParCache(filename, parlist)
@@ -516,8 +518,9 @@ class IrafParList(taskpars.TaskPars):
 
     def Update(self):
         """Check to make sure this list is in sync with parameter file"""
-        self.__pars, self.__pardict, self.__psetlist = \
+        self.__pars, self.__pardict, self.__psets2merge = \
                 self.__filecache.get()
+        if self.__psets2merge: self.__addPsetParams()
 
     def setFilename(self, filename):
         """Change filename and create ParCache object
@@ -557,19 +560,27 @@ class IrafParList(taskpars.TaskPars):
         2013 that this is not happening correctly, and may be an unsafe plan.
         Therefore the code was changed to allow clients to access both copies;
         see getParObjects() and any related code. """
-        # return immediately if they have already been added
-        if self.__psetlist is None: return
-        # otherwise...
-        self.__hasPsets = True
+        # return immediately if they have already been added or
+        # if we are in the midst of a recursive call tree
+        if self.__psetLock or self.__psets2merge is None:
+            return
+        # otherwise, merge in any PSETs
+        if len(self.__psets2merge) > 0:
+            self.__hasPsets = True # never reset
+        self.__psetLock = True # prevent us from coming in recursively
+
         # Work from the pset's pardict because then we get
         # parameters from nested psets too
-        for p in self.__psetlist:
+        for p in self.__psets2merge:
             # silently ignore parameters from psets that already are defined
             psetdict = p.get().getParDict()
             for pname in psetdict.keys():
                 if not self.__pardict.has_exact_key(pname):
                     self.__pardict.add(pname, psetdict[pname])
-        self.__psetlist = None
+
+        # back to normal state
+        self.__psets2merge = None
+        self.__psetLock = False
 
     def addParam(self, p):
         """Add a parameter to the list"""
@@ -605,13 +616,14 @@ class IrafParList(taskpars.TaskPars):
         self.__pardict.add(p.name, p)
         if isinstance(p, IrafParPset):
             # parameters from this pset will be added too
-            if self.__psetlist is None:
+            if self.__psets2merge is None:
                 # add immediately
-                self.__psetlist = [p]
+                self.__psets2merge = [p]
                 self.__addPsetParams()
             else:
                 # just add to the pset list
-                self.__psetlist.append(p)
+                self.__psets2merge.append(p)
+                # can't call __addPsetParams here as we may now be inside a call
 
     def isConsistent(self, other):
         """Compare two IrafParLists for consistency
@@ -693,7 +705,7 @@ class IrafParList(taskpars.TaskPars):
 
     def hasPar(self,param):
         """Test existence of parameter named param"""
-        if self.__psetlist: self.__addPsetParams()
+        if self.__psets2merge: self.__addPsetParams()
         param = irafutils.untranslateName(param)
         return param in self.__pardict
 
@@ -711,7 +723,7 @@ class IrafParList(taskpars.TaskPars):
             return self.__pars
 
     def getParDict(self):
-        if self.__psetlist: self.__addPsetParams()
+        if self.__psets2merge: self.__addPsetParams()
         return self.__pardict
 
     def getParObject(self,param):
@@ -720,7 +732,7 @@ class IrafParList(taskpars.TaskPars):
         any duplicated PSET pars via __addPsetParams), but does not look
         down into PSETs. Note the difference between this and getParObjects
         in their different return types. """
-        if self.__psetlist: self.__addPsetParams()
+        if self.__psets2merge: self.__addPsetParams()
         try:
             param = irafutils.untranslateName(param)
             return self.__pardict[param]
@@ -728,13 +740,13 @@ class IrafParList(taskpars.TaskPars):
             raise e.__class__("Error in parameter '" +
                     param + "' for task " + self.__name + "\n" + str(e))
 
-    def getParObjects(self, param):
+    def getParObjects(self, param, typecheck=True):
         """
-        Returns _all_ IrafPar objects matching the name given (param),
+        Returns all IrafPar objects matching the string name given (param),
         in the form of a dict like:
             { scopename : <IrafPar instance>, ... }
-        where scope is '' if the par was found as a regular par in this list,
-        or, where scope is psetname if the par was found inside a PSET.
+        where scopename is '' if par was found as a regular par in this list,
+        or, where scopename is psetname if the par was found inside a PSET.
         It is possible that some dict values will actually be the same object
         in memory (see docs for __addPsetParams).
 
@@ -742,10 +754,28 @@ class IrafParList(taskpars.TaskPars):
         found at the "top level" (a regular par inside this par list)
         even if it is also in a PSET.
 
+        typecheck: If multiple par objects are found, and typecheck is set to
+        True, only the first (e.g. top level) will be returned if those
+        par objects have a different value for their .type attribute.
+        Otherwise all par objects found are returned in the dict.
+
         Note the difference between this and getParObject in their
         different return types.
         """
-        if self.__psetlist: self.__addPsetParams()
+        # Notes:
+        # To accomplish the parameter setting (e.g. setParam) this calls up
+        # all possible exact-name-matching pars in this par list, whether
+        # they be on the "top" level with that name (param), or down in some
+        # PSET with that name (param).  If we are simply an IRAFish task, then
+        # this is fine as we can assume the task likely does not have a par of
+        # its own and a PSET par, both of which have the same name. Thus any
+        # such case will acquire a copy of the PSET par at the top level. See
+        # discussion of this in __addPsetParams().
+        # BUT, if we are a CL script (e.g. mscombine.cl), we could have local
+        # vars which happen to have the same names as PSET pars.  This is an
+        # issue that we need to handle and be aware of (see typecheck arg).
+
+        if self.__psets2merge: self.__addPsetParams()
         param = irafutils.untranslateName(param)
         retval = {}
 
@@ -764,17 +794,24 @@ class IrafParList(taskpars.TaskPars):
         # There is a PSET in here somewhere...
         allpsets = [p for p in self.__pars if isinstance(p, IrafParPset)]
         for pset in allpsets:
-            # Search its pars.  We definitely do NOT want a copy,
+            # Search the pset's pars.  We definitely do NOT want a copy,
             # we need the originals to edit.
-            its_plist = pset.get().getParList(docopy=0)
-            # might need to change next line to use minmatch dict
-            matching_pars = [pp for pp in its_plist if pp.name.startswith(param)]
+            its_task = pset.get()
+            its_plist = its_task.getParList(docopy=0)
+            # assume full paramname given (no min-matching inside of PSETs)
+            matching_pars = [pp for pp in its_plist if pp.name == param]
             if len(matching_pars) > 1:
                 raise RuntimeError('Unexpected multiple matches for par: '+ \
-                                   param+', are: '+str(matching_pars))
-            # add it to outgoing dict
+                                   param+', are: '+str([p.name for p in matching_pars]))
+            # found one with that name; add it to outgoing dict
             if len(matching_pars) > 0:
-                retval[pset.name] = matching_pars[0]
+                addit = True
+                if typecheck and '' in retval:
+                    # in this case we already found a top-level and we've been
+                    # asked to make sure to return only same-type matches
+                    addit = matching_pars[0].type == retval[''].type # attr is a char
+                if addit:
+                    retval[pset.name] = matching_pars[0]
         return retval
 
     def getAllMatches(self,param):
@@ -809,7 +846,8 @@ class IrafParList(taskpars.TaskPars):
         """Set task parameter 'param' to value (with minimum-matching).
            scope, idxHint, and check are included for use as a task object
            but they are currently ignored."""
-        for par_obj in self.getParObjects(param).values():
+        matches_dict = self.getParObjects(param)
+        for par_obj in matches_dict.values():
             par_obj.set(value)
 
     def setParList(self,*args,**kw):
