@@ -4,7 +4,7 @@ These are python stubs that are overloaded by a c version implementations.
 If the c versions do not exist, then these routines will do nothing
 
 
-$Id: wutil.py 1793 2012-07-10 15:49:32Z sontag $
+$Id: wutil.py 2069 2013-10-09 15:33:36Z sontag $
 """
 from __future__ import division # confidence high
 
@@ -57,8 +57,8 @@ if WUTIL_ON_MAC and not _skipDisplay:
 GRAPHICS_ALWAYS_ON_TOP = 'PYRAF_GRAPHICS_ALWAYS_ON_TOP' in os.environ
 
 # attempt to override with xutil or aqua versions
-_hasAqua = 0
-_hasXWin = 0
+_has_aqutil = 0
+_has_xutil = 0
 try:
     if WUTIL_USING_X and not _skipDisplay:
         # set an env var before importing xutil (see PyRAF FAQ on this)
@@ -77,7 +77,7 @@ try:
         # Successful intialization. Reset dummy methods with
         # those from 'xutil' now.
         from pyraf.xutil import *
-        _hasXWin = 1 # Flag to mark successful initialization of XWindow
+        _has_xutil = 1 # Flag to mark successful initialization of XWindow
         closeGraphics = closeXGraphics
 
     else:
@@ -92,15 +92,16 @@ try:
                 # override the few Mac-specific functions needed
                 from aqutil import getFocalWindowID, setFocusTo, getParentID
                 from aqutil import moveCursorTo, getPointerPosition
-                _hasAqua = 1
+                _has_aqutil = 1
             except:
-                _hasAqua = 0
+                _has_aqutil = 0
                 print "Could not import aqutil, please see the online PyRAF FAQ"
+                print "http://www.stsci.edu/institute/software_hardware/pyraf/pyraf_faq#5.13"
 
 except ImportError:
-    _hasXWin = 0 # Unsuccessful init of XWindow
+    _has_xutil = 0 # Unsuccessful init of XWindow
 except EnvironmentError:
-    _hasXWin = 0 # Unsuccessful init of XWindow
+    _has_xutil = 0 # Unsuccessful init of XWindow
 
 # Clean up the namespace a bit...
 try:
@@ -151,7 +152,7 @@ def getTopID(WindowID):
 
     # a "top ID" makes less sense if we are not using X
     if not WUTIL_USING_X:
-        if _hasAqua:
+        if _has_aqutil:
             return aqutil.getTopIdFor(wid)
         else:
             return wid # everything is its own top
@@ -175,7 +176,7 @@ def forceFocusToNewWindow():
     actually in the front, where focus would be.  With X, any new window
     comes to the front anyway, so this is a no-op.  Currently this is
     only necessary under Aqua. """
-    if _hasAqua:
+    if _has_aqutil:
         aqutil.focusOnGui()
 
 def isViewable(WindowID):
@@ -256,7 +257,7 @@ class TerminalFocusEntity(FocusEntity):
             self.windowID = getFocalWindowID()
             if self.windowID == -1:
                 self.windowID = None
-            if _hasAqua:
+            if _has_aqutil:
                 scrnPosDict = aqutil.getPointerGlobalPosition()
                 self.lastScreenX = scrnPosDict['x']
                 self.lastScreenY = scrnPosDict['y']
@@ -277,7 +278,7 @@ class TerminalFocusEntity(FocusEntity):
         if self.windowID == getFocalWindowID():
             # focus is already here
             return
-        if _hasAqua:
+        if _has_aqutil:
             if self.lastScreenX is not None and cursorToo:
                 moveCursorTo(self.windowID, self.lastScreenX, self.lastScreenY,
                              0, 0)
@@ -290,7 +291,7 @@ class TerminalFocusEntity(FocusEntity):
     def saveCursorPos(self):
         if (not self.windowID) or (self.windowID != getFocalWindowID()):
             return
-        if _hasAqua:
+        if _has_aqutil:
             scrnPosDict = aqutil.getPointerGlobalPosition()
             self.lastScreenX = scrnPosDict['x']
             self.lastScreenY = scrnPosDict['y']
@@ -494,25 +495,82 @@ terminal = TerminalFocusEntity()
 focusController = FocusController(terminal)
 
 # debug helper
-def dumpspecs():
+def dumpspecs(outstream = None, skip_volatiles = False):
     """ Dump various flags, settings, values to the terminal.  This is not to
     be used internal to this module - it must wait until the module is fully
-    imported for all the values to be finalized. """
-    print "platform = "+str(sys.platform)
-    print "PY3K = "+str(capable.PY3K)
-    print "c.OF_GRAPHICS = "+str(capable.OF_GRAPHICS)
-    if not capable.OF_GRAPHICS: return
-    print "TclVersion = "+str(capable.Tkinter.TclVersion)
-    print "TkVersion = "+str(capable.Tkinter.TkVersion)
-    print "is_darwin_and_x = "+str(capable.is_darwin_and_x())
-    print "which_darwin_linkage = "+str(capable.which_darwin_linkage())
-    print "WUTIL_ON_MAC = "+str(WUTIL_ON_MAC)
-    print "WUTIL_ON_WIN = "+str(WUTIL_ON_WIN)
-    print "WUTIL_USING_X = "+str(WUTIL_USING_X)
-    print "skipDisplay = "+str(_skipDisplay)
-    print "hasGraphics = "+str(hasGraphics)
-    print "hasAqua = "+str(_hasAqua)
-    print "hasXWin = "+str(_hasXWin)
+    imported for all the values to be finalized.  If outstream is not given,
+    this will simply dump to sys.stdout. """
+
+    pyrver = 'unknown'
+    try:
+       from pyraf import __version__ as pyrver
+    except:
+       pass
+
+    out = "python exec = "+str(sys.executable)
+    out += "\npython ver = "+str(sys.version)
+    out += "\nplatform = "+str(sys.platform)
+    if not skip_volatiles:
+        out += "\nPyRAF ver = "+pyrver
+    out += "\nPY3K = "+str(capable.PY3K)
+    out += "\nc.OF_GRAPHICS = "+str(capable.OF_GRAPHICS)
+    if hasattr(capable, 'get_dc_owner'): # rm hasattr at/after v2.2
+        if skip_volatiles:
+            out +="\n/dev/console owner = <skipped>"
+        else:
+            out +="\n/dev/console owner = "+str(capable.get_dc_owner(False, True))
+
+    if not capable.OF_GRAPHICS:
+        if hasattr(capable, 'TKINTER_IMPORT_FAILED'):
+            out += "\nTkinter import failed."
+        else:
+            out += "\nTkinter use unattempted."
+    else:
+        out += "\nTclVersion = "+str(capable.Tkinter.TclVersion)
+        out += "\nTkVersion = "+str(capable.Tkinter.TkVersion)
+        out += "\nWUTIL_ON_MAC = "+str(WUTIL_ON_MAC)
+        out += "\nWUTIL_ON_WIN = "+str(WUTIL_ON_WIN)
+        out += "\nWUTIL_USING_X = "+str(WUTIL_USING_X)
+        out += "\nis_darwin_and_x = "+str(capable.is_darwin_and_x())
+        if WUTIL_ON_MAC:
+            out += "\nwhich_darwin_linkage = "+str(capable.which_darwin_linkage())
+        else:
+            out += "\nwhich_darwin_linkage = (not darwin)"
+        out += "\nskip display = "+str(_skipDisplay)
+        out += "\nhas graphics = "+str(hasGraphics)
+        out += "\nimported aqutil = "+str(bool(_has_aqutil))
+        out += "\nimported xutil = "+str(bool(_has_xutil))
+
+        if 'PYRAFGRAPHICS' in os.environ:
+            val = os.environ['PYRAFGRAPHICS']
+            out += "\nPYRAFGRAPHICS = "+val
+            if val == "matplotlib":
+                mpl_ok = False
+                try:
+                    import matplotlib as mpl
+                    mpl_ok = True
+                except:
+                    out += "\nCannot import matplotlib"
+                if mpl_ok:
+                    if hasattr(mpl, 'tk_window_focus'):
+                        out += "\nmpl.tk_window_focus = "+str(mpl.tk_window_focus())
+                    else:
+                        out += "\nmpl.tk_window_focus = function not supported"
+                    mpldir = os.path.split(mpl.__file__)[0]
+                    import glob
+                    flist = glob.glob(mpldir+os.path.sep+'backends'+os.path.sep+'*.so')
+                    flist = [os.path.split(f)[1] for f in flist]
+                    out += "\nmpl backends = "+str(flist)
+                    tkaggbknd = mpldir+'/backends/_tkagg.so'
+                    if os.path.exists(tkaggbknd):
+                        out += "\ntry: /usr/bin/otool -L "+tkaggbknd
+        else:
+            out += "\nPYRAFGRAPHICS not set"
+
+    if outstream:
+        outstream.write(out+'\n')
+    else:
+        print out
 
 
 
@@ -532,7 +590,7 @@ if _skipDisplay:
             if hasattr(capable, 'TKINTER_IMPORT_FAILED'):
                 print "Tkinter import failed."
 else:
-    if _hasXWin or _hasAqua:
+    if _has_xutil or _has_aqutil:
         hasGraphics = focusController.hasGraphics
     elif WUTIL_ON_MAC:
         # Handle case where we are on the Mac with no X and no PyObjc.  We can
@@ -549,4 +607,5 @@ else:
         print "No graphics display available for this session."
         print "Graphics tasks that attempt to plot to an interactive " + \
                           "screen will fail."
+        print 'For help, search "PyRAF FAQ 5.13" or contact "help@stsci.edu".'
         print ""
